@@ -19,6 +19,7 @@ SOURCE_CHECK_ACTIONS = [
 ]
 PRE_DEPLOY_ACTIONS = []
 PRE_INSTALL_ACTIONS = []
+POST_RSYNC_ACTIONS = []
 POST_INSTALL_ACTIONS = []
 POST_DEPLOY_ACTIONS = []
 
@@ -32,3 +33,8 @@ GIT_IGNORE_PATH = '.gitignore'
 #-----------------------------------------------------------------------------------------------------------------------
 WEB_SERVERS = []  # required
 REMOTE_ENVIRONMENTS = {}
+
+# Performance
+#-----------------------------------------------------------------------------------------------------------------------
+PROCESSES = 2
+THREADS = 20
