@@ -184,8 +184,8 @@ def deploy(settings_path=None, **settings_patches):
 def deploy_remote_core():
     """ The part to run on each server
     """
-    host_name = env.host_string.split('@')[-1]
-    print section_title('Deploy to %s' % host_name)
+    env.host_name = host_name = env.host_string.split('@')[-1]
+    print section_title('Deploy to %s' % white(host_name))
 
     # Find Jack first
     with settings(hide('everything'), warn_only=True):
@@ -198,22 +198,47 @@ def deploy_remote_core():
     # Get/Set variables
     beanstalk_settings = env.beanstalk_settings
     project_name = beanstalk_settings['PROJECT_NAME']
+    verbose_level = beanstalk_settings['VERBOSE']
 
     # Create this app to server
-    run('bsjack server.create_app:%s,VERBOSE=%d' % (project_name, beanstalk_settings['VERBOSE']))
+    print section_title('Create app if it\'s not existed.')
+    run('bsjack server.create_app:%s,VERBOSE=%d --hide=status' % (project_name, beanstalk_settings['VERBOSE']))
 
     # Call pre-install script
     run_hooked_actions('PRE_INSTALL_ACTIONS')
 
     # Get app info first
-    app_info = json.loads(run('bsjack server.app_info:%s,VERBOSE=%d,USE_JSON=True' %
-                              (project_name, beanstalk_settings['VERBOSE'])))
-    print app_info
+    with hide('everything'):
+        app_info = json.loads(run('bsjack server.app_info:%s,VERBOSE=%d,USE_JSON=True --hide=status' %
+                                  (project_name, beanstalk_settings['VERBOSE'])))
+    env.remote_app_info = app_info
+    remote_base_path = app_info['path']['base']
+    remote_source_path = app_info['path']['source']
+
+    print section_title('Send codebase to remote: %s' % white(remote_base_path))
     # Send code
-    pass
+    rsync_argument = '-DLhrctz --delete --delete-excluded'
+    rsync_argument += ' --exclude \'*.pyc\' --exclude \'*.pyo\' --exclude-from \'.gitignore\''
+    if verbose_level == 0:
+        rsync_argument += ' -q'
+    elif verbose_level == 2:
+        rsync_argument += ' -v'
+    elif verbose_level == 3:
+        rsync_argument += ' -Pv'
+    target = 'beanstalk@%s:%s' % (host_name, remote_source_path)
+    local(rsync(env.temp_directory, target, rsync_argument=rsync_argument, expand_to_destination=True))
+
+    # Call post-install script
+    run_hooked_actions('POST_RSYNC_ACTIONS')
+
+    # Update this app in server
+    print section_title('Update app in server')
+    run('bsjack server.update_app:%s,VERBOSE=%d --hide=status' % (project_name, beanstalk_settings['VERBOSE']))
 
     # Call post-install script
     run_hooked_actions('POST_INSTALL_ACTIONS')
 
-    # Update this app in server
-    run('bsjack server.update_app:%s,VERBOSE=%d' % (project_name, beanstalk_settings['VERBOSE']))
+    # Commit state to git repo
+    print section_title('Commit current code to git repo')
+    run('bsjack server.commit_app:%s,processes=%d,threads=%d,VERBOSE=%d --hide=status' %
+        (project_name, beanstalk_settings['PROCESSES'], beanstalk_settings['THREADS'], beanstalk_settings['VERBOSE']))
