@@ -1,5 +1,7 @@
+import datetime
 import json
 import os
+import socket
 from fabric.api import *
 from fabric.colors import *
 from fabric.contrib.console import confirm
@@ -37,13 +39,23 @@ def project_venv_path(project_name):
     return os.path.join(project_base, 'venv/')
 
 
+def project_uwsgi_path(project_name):
+    project_base = project_base_path(project_name)
+    return os.path.join(project_base, 'uwsgi.ini')
+
+
+def apache_conf_path():
+    return '/etc/httpd/conf.d/beanstalk.conf'
+
+
 def apps_pool_path():
     beanstalk_settings = env.get('beanstalk_settings', None)
     if beanstalk_settings is None:
         abort('Load beanstalk settings first')
 
     beanstalk_stack_base = beanstalk_settings['BEANSTALK_STACK_BASE']
-    return os.path.join(beanstalk_stack_base, 'apps')
+    return os.path.join(beanstalk_stack_base, 'apps.json')
+
 
 # Tasks
 #-----------------------------------------------------------------------------------------------------------------------
@@ -137,6 +149,10 @@ def setup(settings_path=None, **settings_patches):
     local('sed -i.bak -e "/^AllowGroups wheel$/cAllowGroups wheel beanstalk" /etc/ssh/sshd_config')
     local('/etc/init.d/sshd restart')
 
+    # Apache conf
+    local('touch %s' % apache_conf_path())
+    local('chown beanstalk:beanstalk %s' % apache_conf_path())
+
     # Create homebase
     print section_title('Deploy target is: %s' % beanstalk_stack_base)
     local('mkdir -p %s' % beanstalk_stack_base)
@@ -162,6 +178,7 @@ def create_app(project_name, settings_path=None, **settings_patches):
     project_venv = project_venv_path(project_name)
 
     if os.path.exists(project_base):
+        print 'Nothing to do. App existed!'
         return
 
     # Create folders
@@ -186,9 +203,32 @@ def create_app(project_name, settings_path=None, **settings_patches):
             local('git commit -m "Init commit"')
 
     # Add to pool
+    uwsgi_port = None
+
     def editor(apps_pool):
         # Get uwsgi port
-        uwsgi_port = 50000
+        global uwsgi_port
+        if project_name in apps_pool:
+            uwsgi_port = apps_pool[project_name]['uwsgi']['port']
+        else:
+            app_ports = [app['uwsgi']['port'] for name, app in apps_pool.items()]
+            if len(app_ports) == 0:
+                uwsgi_port = 50000  # >= 49152
+            else:
+                try_times = 1
+                while try_times <= 10:
+                    try:
+                        uwsgi_port = max(app_ports) + try_times
+                        s = socket.socket()
+                        s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+                        s.bind(('localhost', uwsgi_port))
+                        break
+                    except socket.error:
+                        uwsgi_port = None
+                        try_times += 1
+        if uwsgi_port is None:
+            abort('No available port')
+
         # Add it
         apps_pool[project_name] = {
             'path': {
@@ -203,8 +243,29 @@ def create_app(project_name, settings_path=None, **settings_patches):
         }
 
         return apps_pool
+    print section_title('Add app to app_pool')
     edit_file(apps_pool_path(), editor=editor)
 
+    print section_title('Add apache to apache httpd')
+    with open(apache_conf_path(), 'r') as f:
+        apache_conf = f.read()
+    if not '# %s' % project_name in apache_conf:
+        commands = [
+            '# %s' % project_name,
+            '#' + '-' * 20,
+            '<Location /%s>' % project_name,
+            '    SetHandler uwsgi-handler',
+            '    uWSGISocket 127.0.0.1:%s' % uwsgi_port,
+            '</Location>',
+            '#' + '-' * 20,
+            ' ',
+        ]
+        sed_commands = []
+        for command in commands:
+            sed_commands.append('$a\\\n%s' % command)
+        sed_command = ';'.join(sed_commands)
+        local('sed -i.bak -e \'%s\' %s && rm -rf %s.bak' % (sed_command, apache_conf_path(), apache_conf_path()))
+
 
 @task
 @beanstalk_role('server')
@@ -241,8 +302,17 @@ def update_app(project_name, settings_path=None, **settings_patches):
     load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
     set_verbose_level()
 
-    print 'update_app: %s' % project_name
-    # TODO: update app
+    project_source = project_source_path(project_name)
+    project_venv = project_venv_path(project_name)
+
+    print section_title('Install/Update packages in virtualenv')
+    with settings(virtual_env(project_venv)):
+        local('pip install -r %s' % os.path.join(project_source, 'requirements.txt'))
+
+    print section_title('Compile python code')
+    local('find %s -name \'*.pyc\' -exec rm -rf {} \;' % project_source)
+    local('find %s -name \'*.pyo\' -exec rm -rf {} \;' % project_source)
+    local('python -m compileall -q %s' % project_source)
 
 
 @task
@@ -255,16 +325,7 @@ def reload_app(project_name, settings_path=None, **settings_patches):
     set_verbose_level()
 
     print section_title('Reload app: %s' % project_name)
-
-    project_base = project_base_path(project_name)
-    print project_base
-    # TODO: touch the uwsgi.ini
-
-
-# disable_app
-
-
-# enable_app
+    local('touch %s' % project_uwsgi_path(project_name))
 
 
 @task
@@ -305,7 +366,14 @@ def app_info(project_name, settings_path=None, **settings_patches):
         apps_pool = json.loads(f.read())
 
     # Gather information
-    apps = apps_pool[project_name]
+    if project_name in apps_pool:
+        apps = apps_pool[project_name]
+    else:
+        if beanstalk_settings.get('USE_JSON', False):
+            print json.dumps({'error': 'No such app'})
+        else:
+            print red('No such app')
+        return
 
     if beanstalk_settings.get('USE_JSON', False):
         result = apps
@@ -321,6 +389,8 @@ def app_info(project_name, settings_path=None, **settings_patches):
         print 'Source : %s' % apps['path']['source']
         print 'Logs   : %s' % apps['path']['logs']
         print 'venv   : %s' % apps['path']['venv']
+        print '-' * 40
+        print 'uwsgi port : %s' % apps['uwsgi']['port']
 
 
 @task
@@ -331,3 +401,26 @@ def list_apps(settings_path=None, **settings_patches):
     settings_patches = eval_kwargs(settings_patches)
     load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
     set_verbose_level()
+
+    with open(apps_pool_path(), 'r') as f:
+        apps_pool = json.loads(f.read())
+    apps = [key for key, value in apps_pool.items()]
+
+    print section_title('Beanstalk-stack apps')
+    print '-=' * 40
+    for app in apps:
+        print app
+
+
+@task
+@beanstalk_role('server')
+def commit_app(project_name, processes=2, threads=20, settings_path=None, **settings_patches):
+    """Get list of apps
+    """
+    settings_patches = eval_kwargs(settings_patches)
+    load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
+    set_verbose_level()
+
+    project_source = project_source_path(project_name)
+    with settings(lcd(project_source), hide('warnings'), warning=True):
+        local('git add -A && git commit -m "state at %s"' % datetime.datetime.now().strftime('%s'))
